$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST

;*********************************************************************
;THIS PROGRAM IS WRITTEN TO RECIEVE A TRANSMISSION IN SDLC MODE UNDER 
;DMA CONTROL AND ASSUMES ALL RECEPTIONS WILL BE VALID, THUS THE NAME 
;Recieve Valid Under Dma Control (RVUDC1).  
;
;ASSUMPTIONS:
;   THE DESTINATION OF RECEIVED DATA IS IN EXTERNAL RAM AND THE STARTING 
;   ADDRESS IS DEFINED BY:
;               MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;   THE STARTING LOCATION WHERE THE DATA IS STORED WILL ALWAYS BEGIN AT 
;   THE SAME ADDRESS FOR EACH NEW PACKET (THE OLD DATA IS WRITTEN OVER)
;       THE NUMBER OF BYTES RECEIVED IS STORED AT LOCATIONS 400H AND 401H
;       THE MESSAGE STARTS AT LOCATION 402H
;
;   8-BIT ADDRESSING
;
;   16-BIT CRC
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;
;
;SFRS USED AND MODIFIED:
;           ACC
;           ADR0
;           ADR1
;           ADR2
;           ADR3
;           BAUD
;           BCRH1
;           BCRL1
;           DARH1
;           DARL1
;           DCON1
;           DPL
;           DPH
;           GMOD
;           IE          
;           IEN1
;           RFIFO
;           RSTAT
;           SARL1
;           TSTAT
;
;BITS USED AND MODIFIED:
;           C
;
;*********************************************************************

$EJ

;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN UNDER THE ASSUMPTION THAT NO RECEIVE ERRORS 
;WILL EVER OCCUR, HOWEVER, TO KEEP THE PROGRAM FROM HANGING UP IF AN 
;ERROR DOES OCCUR, SECTIONS OF CODE WERE ADDED.  THESE SECTIONS OF CODE 
;ARE IDENTIFIED BY THE PHRASE "EXTRA CODE FOR ERROR CONDITION" BUT THESE 
;SHOULD NOT BE USED AS THEY HAVE NOT BEEN THOROUGHLY CHECKED OUT.  ALL 
;REFERENCES TO ANY TYPE OF ERROR(S) APPLY TO THOSE ERRORS THAT ARE DETECTED 
;AND FLAGGED BY THE GSC HARDWARE.  SDLC WAS CHOSEN AND IN THIS PROTOCOL
;RECEIVER ERRORS CAN ALWAYS OCCUR.  FOR THIS REASON THE PROGRAM WILL NOT
;FUNCTION "AS-IS" IN A REAL APPLICATION AND WILL HAVE TO BE MODIFIED TO
;HANDLE THE ERRORS THAT MAY OCCUR.  THE PROGRAM, REUDC1, HAS BEEN MODIFIED
;PROPERLY HANDLE RECEIVER ERRORS AND BEEN VERIFIED TO FUNCTION PROPERLY.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT HOPEFULLY
;IS SIMPLE ENOUGH FOR USERS WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ

MSG_DEST            EQU 402H        ;STARTING LOCATION OF RECEIVED
                                    ;DATA

ADDRESS0            EQU     11H
ADDRESS1            EQU     ADDRESS0
ADDRESS2            EQU     ADDRESS0
ADDRESS3            EQU     ADDRESS0

$EJ

ORG 0
    JMP INIT1

ORG 2BH
REC_VALID:
    JMP VALID_REC

ORG 33H
REC_ERROR:
    JMP ERROR_REC

$EJ

ORG 100H
VALID_REC:

;*********************************************************************
; (D) - REINITIALIZE DMA - STEPS A AND B WERE OMITTED BECAUSE THE CHECK
;   FOR PROPER MESSAGE LENGTHS BELONGS ABOVE LAYER 2 IN THE ISO MODEL
;   WHICH THIS PROGRAM IS NOT MEANT TO ADDRESS - STEP C WAS NOT DONE
;   BECAUSE THE DATA IS NOT MOVED BUT WRITTEN OVER INSTEAD. STEP A WAS
;   TO CHECK MESSAGE SIZE, STEP B WAS TO NOTIFY ORIGINATOR THAT MESSAGE
;   SIZE WAS NOT CORRECT, AND STEP C WAS TO MOVE/EXAMINE DATA
;*********************************************************************

    MOV DPTR,#(MSG_DEST - 2)
    MOV A,BCRH1
    CPL A                           ;CPL BECAUSE BYTE COUNT STARTS AT
                                    ;0FFFFH AND COUNTS DOWN
    MOVX @DPTR,A                    ;STORE HIGH BYTE OF # BYTES RECEIVED
    INC DPTR
    MOV A,BCRL1
    CPL A                           ;CPL BECAUSE BYTE COUNT STARTS AT 
                                    ;0FFFFH AND COUNTS DOWN
    MOVX @DPTR,A                    ;STORE LOW BYTE OF # BYTES RECEIVED

    MOV DARL1,#LOW (MSG_DEST)
    MOV DARH1,#HIGH (MSG_DEST)      ;SETUP DESTINATION ADDRESS

    MOV BCRH1,#0FFH
    MOV BCRL1,#0FFH                 ;INIT BYTE COUNT FOR MAX LENGTH

;*********************************************************************
; (E) - ENABLE RECEIVER AND SET GO BIT
;*********************************************************************

    ORL DCON1,#01                   ;SET GO BIT

    SETB GREN                       ;ENABLE RECEIVER

    RETI

$EJ

ORG 200H
INIT2:
;*********************************************************************
; USER GENERIC DMA INITIALIZATION (FOR RECEIVER)
;*********************************************************************
    MOV SARL1,#RFIFO                ;SETUP SOURCE ADDRESS FOR RFIFO

    MOV DARL1,#LOW (MSG_DEST)
    MOV DARH1,#HIGH (MSG_DEST)      ;SETUP DESTINATION ADDRESS

    MOV BCRH1,#0FFH
    MOV BCRL1,#0FFH                 ;INIT BYTE COUNT FOR MAX LENGTH

    MOV DCON1,#01101001B            ;SETUP DMA FOR EXT RAM (DEST), SFR
                                    ;(SOURCE), SERIAL PORT DEMAND MODE,
                                    ;AND SET GO BIT    

;*********************************************************************
; USER GENERIC GSC RECEIVER INITIALIZATION
;*********************************************************************

    MOV BAUD,#60                    ;INIT BAUD RATE FOR # DIVISABLE BY 6

    MOV GMOD,#01                    ;INIT FOR SDLC, 0 PREAMBLE (N/A FOR
                                    ;RECEIVER), 16 CRC, 8 BIT ADDR

    SETB DMA                        ;SETUP INTERRUPTS FOR DMA CONTROL

    MOV ADR0,#ADDRESS0              ;ADDRESSES THAT THIS STATION WILL 
    MOV ADR1,#ADDRESS1              ;RECEIVE (MASK REGS ALREADY = 0
    MOV ADR2,#ADDRESS2              ;FROM RESET)
    MOV ADR3,#ADDRESS3

    SETB GREN                       ;RECIEVER SHOULD ALWAYS BE ENABLED
                                    ;UNLESS DATA ALREADY RECEIVED WOULD
                                    ;BE WRITTEN OVER BEFORE DEALT WITH

    RET

$EJ

ORG 300H
INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION - USER SHOULD PLACE INITIALIZING SOFTWARE
; ROUTINES HERE
;*********************************************************************

    SETB EGSRV                      ;ENABLE RECEIVE VALID INTERRUPT
    SETB EGSRE                      ;ENABLE RECEIVE ERROR INTERRUPT
                                    ;THOUGH IT SHOULD NOT BE USED IN
                                    ;THIS PROGRAM

    SETB EA
    CALL INIT2                      ;GSC INITIALIZATION

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP - USER SHOULD PLACE ALL APPLICATION CODE OR
; BRANCHES TO OTHER SECTIONS OF APPLICATION CODE HERE.
;*********************************************************************

    JMP MAIN                        ;CONTINOUSLY LOOP

;***** EXTRA CODE FOR ERROR CONDITION*****

ERROR_REC:
;*********************************************************************
; THIS SECTION OF CODE WAS ADDED TO KEEP S/W FROM HANGING UP IF AN
; ERROR OCCURS.  THIS HAS NOT BEEN FULLY CHECKED OUT AND SHOULD NOT
; BE USED
;*********************************************************************

    MOV DARL1,#LOW (MSG_DEST)
    MOV DARH1,#HIGH (MSG_DEST)      ;SETUP DESTINATION ADDRESS

    MOV BCRH1,#0FFH
    MOV BCRL1,#0FFH                 ;INIT BYTE COUNT FOR MAX LENGTH

    MOV DCON1,#01101001B            ;SETUP DMA FOR EXT RAM (DEST), SFR
                                    ;(SOURCE), SERIAL PORT DEMAND MODE,
                                    ;AND SET GO BIT    

    SETB GREN                       ;ENABLE RECIEVER, CLEAR RFIFO

    RETI

END
